package io.dcloud.feature.bankcard;

import io.dcloud.common.DHInterface.AbsMgr;
import io.dcloud.common.DHInterface.IApp;
import io.dcloud.common.DHInterface.IFeature;
import io.dcloud.common.DHInterface.ISysEventListener;
import io.dcloud.common.DHInterface.IWebview;
import io.dcloud.common.util.JSUtil;
import io.dcloud.feature.utils.JsonUtils;

import java.io.ByteArrayOutputStream;

import org.json.JSONException;
import org.json.JSONObject;

import android.content.Intent;
import android.graphics.Bitmap;
import android.util.Base64;
import android.util.Log;

import com.idcard.CardInfo;
import com.idcard.TFieldID;
import com.idcard.TRECAPIImpl;
import com.idcard.TStatus;
import com.idcard.TengineID;
import com.turui.bank.ocr.CaptureActivity;

@SuppressWarnings("deprecation")
public class BankCardFeatureImpl implements IFeature,ISysEventListener{
	
	private final static String  TAG = "BankCardFeatureImpl";
	
	public int RESULT_GET_OK = 1;
	public int RESULT_GET_CARD_OK = 2;
	
	private final static int REQUESTCODE = 1;
	private String callBackId;
	private IApp iApp ;
	private IWebview webView;
	
	private JSONObject retJSONObj = null;
	
	private TRECAPIImpl engineDemo = null;
	

	@Override
	public void init(AbsMgr arg0, String arg1) {
		//初始化Feature
		Log.d(TAG , "init");
	}
	
	@Override
	public String execute(IWebview webView, String actionName, String[] args) {
		Log.d(TAG , "execute actionName:"+actionName);
		this.webView = webView;
		
		if("startBankcard".equals(actionName)){
			
			iApp = webView.obtainFrameView().obtainApp();
			callBackId = args[0];//获取callbackId
			iApp.registerSysEventListener(this, SysEventType.onActivityResult);
			
			engineDemo = new TRECAPIImpl();
			TStatus tStatus = engineDemo.TR_StartUP();
			
			if (tStatus == TStatus.TR_TIME_OUT ) {
				//引擎过期
				retJSONObj = JsonUtils.getResultObj("引擎过期",false,null);
        		JSUtil.execCallback(webView, callBackId, retJSONObj, JSUtil.ERROR, false);
			}else  if (tStatus == TStatus.TR_FAIL) {
				//引擎初始化失败
				retJSONObj = JsonUtils.getResultObj("引擎初始化失败",false,null);
        		JSUtil.execCallback(webView, callBackId, retJSONObj, JSUtil.ERROR, false);
			}else{
				CaptureActivity.tengineID = TengineID.TIDBANK;
				CaptureActivity.ShowCopyRightTxt = "";
				//CaptureActivity.SetDisplayLight(true);// 打开闪光灯
				
				Intent intent = new Intent(webView.getActivity(), CaptureActivity.class);
				intent.putExtra("engine", engineDemo);
				webView.getActivity().startActivityForResult(intent, RESULT_GET_OK);
			}
			
		}
		
		return null;
	}
	
	@Override
	public void dispose(String arg0) {
		
	}

	@Override
	public boolean onExecute(SysEventType pEventType, Object pArgs) {
		Object[] _args = (Object[])pArgs; 
        int requestCode = (Integer)_args[0];//获取请求CODE
        int resultCode = (Integer)_args[1];//获取相应CODE
        Intent data = (Intent)_args[2];//返回数据
        
        if(pEventType == SysEventType.onActivityResult){
        	//如果是从其他页面返回的
        	iApp.unregisterSysEventListener(this, SysEventType.onActivityResult);
        	if (requestCode == RESULT_GET_OK && resultCode == RESULT_GET_OK) {
        		// 处理银行卡扫描结果（在界面上显示）
    			CardInfo cardInfo = (CardInfo)data.getSerializableExtra("cardinfo");
    			Bitmap small = CaptureActivity.SmallBitmap;//号码截图
    			Bitmap takeimg = CaptureActivity.TakeBitmap;// 全图
//    			
    			String resultData1 = Base64.encodeToString(Bitmap2Bytes(small),Base64.NO_WRAP);
    			String resultData2 = Base64.encodeToString(Bitmap2Bytes(takeimg),Base64.NO_WRAP);
    			
    			retJSONObj = JsonUtils.getResultObj("识别成功", true);
    			
    			try {
					retJSONObj.put("cardnum", cardInfo.getFieldString(TFieldID.TBANK_NUM));//银行卡号
					retJSONObj.put("cardname", cardInfo.getFieldString(TFieldID.TBANK_NAME));//银行开户行
					retJSONObj.put("cardorgcode", cardInfo.getFieldString(TFieldID.TBANK_ORGCODE));//机构代码
					retJSONObj.put("cardclass", cardInfo.getFieldString(TFieldID.TBANK_CLASS));//获取卡种
					retJSONObj.put("cardCardName", cardInfo.getFieldString(TFieldID.TBANK_CARD_NAME));//获取卡名
					retJSONObj.put("resultData1", resultData1);
					retJSONObj.put("resultData2", resultData2);
//					retJSONObj.put("resultData2", resultData2);
				} catch (JSONException e) {
					e.printStackTrace();
				}
    			
    			JSUtil.execCallback(webView, callBackId, retJSONObj, JSUtil.OK, false);
    			if(null!=engineDemo)
    				engineDemo.TR_ClearUP();
        	}else{
        		retJSONObj = JsonUtils.getResultObj("获取失败",false,null);
        		JSUtil.execCallback(webView, callBackId, retJSONObj, JSUtil.ERROR, false);
        	}
        	
        	
        }
        
		
		return false;
	}
	
	private byte[] Bitmap2Bytes(Bitmap bm) {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		bm.compress(Bitmap.CompressFormat.PNG, 80, baos);
		return baos.toByteArray();
	}
	
}
