package io.dcloud.feature.idcard.ui;

import io.dcloud.feature.utils.ResourceUtils;
import android.app.Activity;
import android.app.FragmentManager;
import android.app.FragmentTransaction;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;

import com.oliveapp.face.livenessdetectionviewsdk.imagecapturer.CaptureImageFragment;
import com.oliveapp.face.livenessdetectionviewsdk.imagecapturer.OnImageCapturedEventListener;
import com.oliveapp.face.livenessdetectorsdk.datatype.AccessInfo;
import com.oliveapp.face.livenessdetectorsdk.utilities.algorithms.DetectedRect;
import com.oliveapp.face.livenessdetectorsdk.utilities.utils.PackageNameManager;

public class IdcardImageCaptureActivity extends Activity implements OnImageCapturedEventListener  {
	
	public static final String TAG = IdcardImageCaptureActivity.class.getSimpleName();

    public static final String EXTRA_CAPTURE_MODE = "capture_mode";

    private CaptureImageFragment mCaptureImageFragment;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		
		if (!PackageNameManager.isPackageNameSet()) {
            PackageNameManager.setPackageName(getPackageName());
        }
        com.oliveapp.camerasdk.utils.PackageNameManager.setPackageName(PackageNameManager.getPackageName());
        super.onCreate(savedInstanceState);
        setContentView(ResourceUtils.getLayoutId(this, "activity_idcard_image_capture"));

        attachUserImageRegistractionFragment();
		
	}
	
	private void attachUserImageRegistractionFragment() {
        FragmentManager mFragmentManager = getFragmentManager();
        CaptureImageFragment userImageRegisterFragment = (CaptureImageFragment) mFragmentManager.findFragmentByTag(CaptureImageFragment.TAG);

        if (userImageRegisterFragment == null) {
            int captureType = getIntent().getExtras().getInt(EXTRA_CAPTURE_MODE, CaptureImageFragment.CAPTURE_MODE_IDCARD_FRONT);
            userImageRegisterFragment = CaptureImageFragment.newInstance(AccessInfo.getInstance(), captureType);

            FragmentTransaction ft = mFragmentManager.beginTransaction();
            ft.add(ResourceUtils.getId(this, "imageCaptureMainLayout"), userImageRegisterFragment, CaptureImageFragment.TAG);
            ft.commit();
        } else {
            if (userImageRegisterFragment.isAdded()) {
                Log.i(TAG, "user image register fragment already attached");
            }
        }
        userImageRegisterFragment.setArgs(this, this);
        mCaptureImageFragment = userImageRegisterFragment;
    }

    @Override
    public void OnImageCaptured(byte[] imageContent, DetectedRect faceRect) {
        boolean faceExists = (faceRect != null);

        Intent i = new Intent();
        i.putExtra("image", imageContent);
        i.putExtra("faceExists", faceExists);
        
        setResult(Activity.RESULT_OK, i);
        finish();
    }

}
