package io.dcloud.feature.touchid;

import io.dcloud.feature.utils.ResourceUtils;
import android.app.Dialog;
import android.content.Context;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.support.v4.hardware.fingerprint.FingerprintManagerCompat;
import android.support.v4.os.CancellationSignal;
import android.util.Log;
import android.view.View;
import android.widget.TextView;


public class TouchIDDialog extends Dialog implements android.view.View.OnClickListener{
	
	private FingerprintManagerCompat manager;
	
	private Context context;
	
	private String title = "提示";
	private String msg = "请验证指纹开通指纹解锁";
	
	private TextView tvAlertTitle,tvAlertMsg,tvAlert;
	
	private TouchIDialogCallback touchIDialogCallback;
	private CancellationSignal cancellationSignal = null;
	private AutoVerifyCount autoVerifyCount = null;

	public TouchIDDialog(Context context) {
		super(context);
		this.context = context;
	}
	
	public TouchIDDialog(Context context, int theme) {
		super(context, theme);
		this.context = context;
	}
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		this.setContentView(ResourceUtils.getLayoutId(context, "layout_touchid_dialog"));
		
		initTouchDialog();
	}
	
	private void initTouchDialog(){
		tvAlertTitle = (TextView)this.findViewById(ResourceUtils.getId(context, "tvAlertTitle"));
		tvAlertMsg = (TextView)this.findViewById(ResourceUtils.getId(context, "tvAlertMsg"));
		tvAlert = (TextView)this.findViewById(ResourceUtils.getId(context, "tvAlert"));
		
		tvAlert.setOnClickListener(this);
		
		tvAlertTitle.setText(title);
		tvAlertMsg.setText(msg);
		
		// 获取一个FingerPrintManagerCompat的实例
        manager = FingerprintManagerCompat.from(context);
        if (cancellationSignal == null) {
		    cancellationSignal = new CancellationSignal();
		}
	}


	@Override
	public void onClick(View v) {
		if(v.getId() == ResourceUtils.getId(context, "tvAlert")){
			if(null!=autoVerifyCount){
				autoVerifyCount.cancel();
				autoVerifyCount = null;
			}
			
			if(null!=cancellationSignal){
				cancellationSignal.cancel();
			}
			
			if(null!=touchIDialogCallback){
				touchIDialogCallback.onCancel(this);
			}
			
			this.dismiss();
		}
		
		
//		switch (v.getId()) {
//		case ResourceUtils.getId(context, "tvAlert"):
//			if(null!=autoVerifyCount){
//				autoVerifyCount.cancel();
//				autoVerifyCount = null;
//			}
//			
//			if(null!=cancellationSignal){
//				cancellationSignal.cancel();
//			}
//			
//			if(null!=touchIDialogCallback){
//				touchIDialogCallback.onCancel(this);
//			}
//			
//			this.dismiss();
//			break;
//		default:
//			break;
//		}
	}
	
	@Override
	public void show() {
		super.show();
		
		manager.authenticate(null, 0, cancellationSignal, new TouchIDAuthenticationCallback(), null);
	}
	
	@Override
	public void dismiss() {
		super.dismiss();
		
		if(null!=cancellationSignal){
			cancellationSignal.cancel();
		}
		
		if(null!=autoVerifyCount){
			autoVerifyCount.cancel();
		}
	}
	
	
	
	
	class TouchIDAuthenticationCallback extends FingerprintManagerCompat.AuthenticationCallback{
		 private static final String TAG = "TouchIDCallback";
		 
		    //当出现错误的时候回调此函数，比如多次尝试都失败了的时候，errString是错误信息
	        @Override
	        public void onAuthenticationError(int errMsgId, CharSequence errString) {
	            Log.d(TAG, "onAuthenticationError: " + errString +" errMsgId:"+errMsgId);
	            //如果验证多次未通过会锁定，锁定后需要30s重试
	            //what == 1
	            sendUIUpdMessage(1,null);
	            if(null!=touchIDialogCallback){
	            	touchIDialogCallback.onError(TouchIDDialog.this,errMsgId,errString);
	            }
	        }

	        // 当指纹验证失败的时候会回调此函数，失败之后允许多次尝试，失败次数过多会停止响应一段时间然后再停止sensor的工作
	        @Override
	        public void onAuthenticationFailed() {
	            Log.d(TAG, "onAuthenticationFailed: " + "验证失败");
	            
	            sendUIUpdMessage(2,null);
	          //what == 2
	        }

	        @Override
	        public void onAuthenticationHelp(int helpMsgId, CharSequence helpString) {
	            Log.d(TAG, "onAuthenticationHelp: " + helpString);
	            //what == 3
	            
	            sendUIUpdMessage(3,null);
	            
	          
//	            switch (helpMsgId) {
//	            case FingerprintManager.FINGERPRINT_ACQUIRED_GOOD:
//	            	tvAlertMsg.setText("图像不清晰");
//	                break;
//	            case FingerprintManager.FINGERPRINT_ACQUIRED_IMAGER_DIRTY:
//	                setResultInfo(R.string.AcquiredImageDirty_warning);
//	                break;
//	            case FingerprintManager.FINGERPRINT_ACQUIRED_INSUFFICIENT:
//	                setResultInfo(R.string.AcquiredInsufficient_warning);
//	                break;
//	            case FingerprintManager.FINGERPRINT_ACQUIRED_PARTIAL:
//	                setResultInfo(R.string.AcquiredPartial_warning);
//	                break;
//	            case FingerprintManager.FINGERPRINT_ACQUIRED_TOO_FAST:
//	                setResultInfo(R.string.AcquiredTooFast_warning);
//	                break;
//	            case FingerprintManager.FINGERPRINT_ACQUIRED_TOO_SLOW:
//	                setResultInfo(R.string.AcquiredToSlow_warning);
//	                break;
//				}
	            
	        }

	        // 当验证的指纹成功时会回调此函数，然后不再监听指纹sensor
	        @Override
	        public void onAuthenticationSucceeded(FingerprintManagerCompat.AuthenticationResult result) {
	            Log.d(TAG, "onAuthenticationSucceeded: " + "验证成功");
	            
	            //what == 4
	            
	            sendUIUpdMessage(4,null);
	            
	            if(null!=touchIDialogCallback){
	            	touchIDialogCallback.onAuthenticationSucceeded(TouchIDDialog.this);
	            }
	        }
	}
	
	class AutoVerifyCount extends CountDownTimer{
		
		Bundle bundle;
		
		public AutoVerifyCount(long millisInFuture, long countDownInterval) {
            super(millisInFuture, countDownInterval);
        }

        @Override
        public void onTick(long millisUntilFinished) {
            //正在计时
        	//what == 5
        	
        	bundle = new Bundle();
        	bundle.putLong("msg", millisUntilFinished/1000);
        	sendUIUpdMessage(5,bundle);
        	
        }

        @Override
        public void onFinish() {
            //计时结束
        	//what == 6
        	sendUIUpdMessage(6,null);
        	manager.authenticate(null, 0, cancellationSignal, new TouchIDAuthenticationCallback(), null);
        }
	}
	
	/**
	 * 发送消息
	 * @param what
	 * @param data
	 * @return
	 */
	private void sendUIUpdMessage(int what,Bundle data){
		Message msg = new Message();
		msg.what = what;
		
		if(null!=data)
			msg.setData(data);
		
		uiUpdHandler.sendMessage(msg);
	}
	
	
	public String getTitle() {
		return title;
	}

	public void setTitle(String title) {
		this.title = title;
	}

	public String getMsg() {
		return msg;
	}

	public void setMsg(String msg) {
		this.msg = msg;
	}

	public TouchIDialogCallback getTouchIDialogCallback() {
		return touchIDialogCallback;
	}

	public void setTouchIDialogCallback(TouchIDialogCallback touchIDialogCallback) {
		this.touchIDialogCallback = touchIDialogCallback;
	}
	
	
	private Handler uiUpdHandler = new Handler(){

		@Override
		public void handleMessage(Message msg) {
			
			if(1 == msg.what){
				//验证错误
				/*tvAlert.setEnabled(false);
	            autoVerifyCount = new AutoVerifyCount(60*1000,1000);
	            autoVerifyCount.start();*/
			}else if(2 == msg.what){
				tvAlertMsg.setText("验证失败，请摆正姿势");
			}else if(3 == msg.what){
				tvAlertMsg.setText("请再试一次");
			}else if(4 == msg.what){
				tvAlertMsg.setText("验证成功");
			}else if(5== msg.what){
				//tvAlertMsg.setText(String.format("%s秒后重新验证",msg.getData().getLong("msg")));
			}else if(6==msg.what){
				/*tvAlert.setEnabled(true);
	        	tvAlertMsg.setText("重新验证指纹");*/
			}
		}
	};
	
	
	

}
