package io.dcloud.feature.touchid;

import io.dcloud.common.DHInterface.AbsMgr;
import io.dcloud.common.DHInterface.IApp;
import io.dcloud.common.DHInterface.IFeature;
import io.dcloud.common.DHInterface.IWebview;
import io.dcloud.common.util.JSUtil;
import io.dcloud.feature.utils.ResourceUtils;

import org.json.JSONObject;

import android.annotation.SuppressLint;
import android.app.Dialog;
import android.app.KeyguardManager;
import android.content.Context;
import android.support.v4.hardware.fingerprint.FingerprintManagerCompat;
import android.util.Log;

@SuppressWarnings("deprecation")
public class TouchIDFeatureImpl implements IFeature{
	
	private final static String  TAG = "TouchIDFeatureImpl";
	
	private FingerprintManagerCompat fingerprintManager = null;
	private KeyguardManager keyguardManager;
	private JSONObject retJSONObj = null;
	
	private TouchIDDialog touchIDDialog;
	
	private String callBackId;
	private IApp iApp ;
	private IWebview webView;
	
	@Override
	public void init(AbsMgr arg0, String arg1) {
		//初始化Feature
		Log.d(TAG , "init");
		
	}
	

	@SuppressLint("NewApi")
	@Override
	public String execute(final IWebview webView, String actionName, String[] args) {
		this.webView = webView;
		fingerprintManager = FingerprintManagerCompat.from(webView.getActivity());
		keyguardManager=(KeyguardManager )webView.getContext().getSystemService(Context.KEYGUARD_SERVICE);
		
		if("deviceVerify".equals(actionName)){
			int version = android.os.Build.VERSION.SDK_INT; 
			
			if(version>=23){
				if (fingerprintManager.isHardwareDetected()) {
					retJSONObj = getResultObj("存在设备", true);
				}else{
					retJSONObj = getResultObj("未发现设备设备", false);
				}
			}else{
				retJSONObj = getResultObj("系统未遵循android标准", false);
			}
			
			return JSUtil.wrapJsVar(retJSONObj);
		}else if("settingVerify".equals(actionName)){
			
			if(!keyguardManager.isKeyguardSecure()){
				//判断用户是已经开启锁屏密码
				retJSONObj = getResultObj("未开启锁屏密码，请设置", false);
			}else if (!fingerprintManager.hasEnrolledFingerprints()) {
				retJSONObj = getResultObj("未设置指纹", false);
			}else{
				retJSONObj = getResultObj("验证通过", true);
			}
			
			return JSUtil.wrapJsVar(retJSONObj);
		}else if("startVerify".equals(actionName)){
			try {
				
				iApp = webView.obtainFrameView().obtainApp();
				callBackId = args[0];//获取callbackId
				
				touchIDDialog = new TouchIDDialog(webView.getActivity(),ResourceUtils.getStyleId(webView.getContext(), "AlertActivity_AlertStyle"));
				touchIDDialog.setTitle("“鲸钱包”的指纹认证");
				touchIDDialog.setMsg("请验证已有手机指纹");
				touchIDDialog.setCancelable(false);
				touchIDDialog.setCanceledOnTouchOutside(false);
				touchIDDialog.setTouchIDialogCallback(new TouchIDialogCallback() {
					
					@Override
					public void onAuthenticationSucceeded(Dialog dialog) {
						
						retJSONObj = getResultObj("指纹验证成功",true);
						dialog.dismiss();
						JSUtil.execCallback(webView, callBackId, retJSONObj, JSUtil.OK, false);
						
					}

					@Override
					public void onCancel(Dialog dialog) {
						retJSONObj = getResultObj("指纹验证取消",false);
						JSUtil.execCallback(webView, callBackId, retJSONObj, JSUtil.OK, false);
					}
					
					@Override
					public void onError(Dialog dialog,int errMsgId,CharSequence errString){
						if(errMsgId == 7){
							dialog.dismiss();
							retJSONObj = getResultObj(errString.toString(),false);
							JSUtil.execCallback(webView, callBackId, retJSONObj, JSUtil.OK, false);
						}
					}
				});
				touchIDDialog.show();
			} catch (Exception e) {
				e.printStackTrace();
				
				retJSONObj = getResultObj("指纹验证失败",false);
				JSUtil.execCallback(webView, callBackId, retJSONObj, JSUtil.OK, false);
			}
		}else if("startVerifyClose".equals(actionName)){
			try {
				
				iApp = webView.obtainFrameView().obtainApp();
				callBackId = args[0];//获取callbackId
				
				touchIDDialog = new TouchIDDialog(webView.getActivity(),ResourceUtils.getStyleId(webView.getContext(), "AlertActivity_AlertStyle"));
				touchIDDialog.setTitle("“鲸钱包”的指纹认证");
				touchIDDialog.setMsg("请验证指纹关闭指纹解锁");
				touchIDDialog.setCancelable(false);
				touchIDDialog.setCanceledOnTouchOutside(false);
				touchIDDialog.setTouchIDialogCallback(new TouchIDialogCallback() {
					
					@Override
					public void onAuthenticationSucceeded(Dialog dialog) {
						
						retJSONObj = getResultObj("指纹验证成功",true);
						dialog.dismiss();
						JSUtil.execCallback(webView, callBackId, retJSONObj, JSUtil.OK, false);
						
					}

					@Override
					public void onCancel(Dialog dialog) {
						retJSONObj = getResultObj("指纹验证取消",false);
						JSUtil.execCallback(webView, callBackId, retJSONObj, JSUtil.OK, false);
					}
					@Override
					public void onError(Dialog dialog,int errMsgId,CharSequence errString){
						
					}
				});
				touchIDDialog.show();
			} catch (Exception e) {
				retJSONObj = getResultObj("指纹验证失败",false);
				JSUtil.execCallback(webView, callBackId, retJSONObj, JSUtil.OK, false);
			}
		}
		return null;
	}
	
	@Override
	public void dispose(String arg0) {
		
	}
	
	
	private JSONObject getResultObj(String msg,boolean state){
		retJSONObj = new JSONObject();
		try {
			retJSONObj.put("message", msg);
			retJSONObj.put("isSuccess", state);
		} catch (Exception e) {
		}
		
		return retJSONObj;
	}

}
