package io.dcloud.feature.verfycode;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.json.JSONObject;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.telephony.SmsMessage;
import android.text.TextUtils;
import android.util.Log;
import io.dcloud.common.DHInterface.AbsMgr;
import io.dcloud.common.DHInterface.IApp;
import io.dcloud.common.DHInterface.IFeature;
import io.dcloud.common.DHInterface.IWebview;
import io.dcloud.common.util.JSUtil;
import io.dcloud.feature.utils.JsonUtils;

/**
 * 验证码自动填充
 * @author yry
 *
 */
public class VerfyCodeFeatureImpl implements IFeature{

	private final static String  TAG = "VerfyCodeFeatureImpl";
	
	private String callBackId;
	private IApp iApp ;
	private IWebview webView;
	private JSONObject retJSONObj = null;
	
	private VerfyCodeBroadcastReceiver smsReceiver;
	private IntentFilter filter;
	
	//private String patternCoder = "(?<!\\d)\\d{4}(?!\\d)";
	private String patternCoder = "\\d+\\.?\\d*";
	private String patternKeyWord = ".*?【鲸钱包】.*?";
	private String patternVerfiyCode = "验证码：.*?\\d+\\.?\\d*";

	@Override
	public void init(AbsMgr arg0, String arg1) {
		Log.d(TAG , "init");
	}
	
	@Override
	public String execute(IWebview webView, String actionName, String[] args) {
		this.webView = webView;
		
		if("verfyCodeInit".equals(actionName)){
			
			smsReceiver = new VerfyCodeBroadcastReceiver();
			filter = new IntentFilter();
			filter.addAction("android.provider.Telephony.SMS_RECEIVED");
			filter.setPriority(Integer.MAX_VALUE);
			webView.getActivity().registerReceiver(smsReceiver, filter);
			
			retJSONObj = JsonUtils.getResultObj("初始化成功", true);
			return JSUtil.wrapJsVar(retJSONObj);
			
		}else if("verfyCodeDestroy".equals(actionName)){
			if(null!=smsReceiver){
				webView.getActivity().unregisterReceiver(smsReceiver);
			}
			retJSONObj = JsonUtils.getResultObj("注销成功", true);
			return JSUtil.wrapJsVar(retJSONObj);
		}
		
		return null;
	}
	
	@Override
	public void dispose(String arg0) {
		if(null!=smsReceiver){
			webView.getActivity().unregisterReceiver(smsReceiver);
		}
	}
	
	class VerfyCodeBroadcastReceiver extends BroadcastReceiver{

		@Override
		public void onReceive(Context context, Intent intent) {
			Object[] objs = (Object[]) intent.getExtras().get("pdus");
			
			for (Object obj : objs) {
				
				byte[] pdu = (byte[]) obj;
				SmsMessage sms = SmsMessage.createFromPdu(pdu);
				String message = sms.getMessageBody();
				String from = sms.getOriginatingAddress();
				
//				Log.d("短信内容", "message：" + message);
//				Log.d("短信来源", "from ：" + from);
				
				boolean isVerfiy = match(patternKeyWord,message);
				//是否存在关键字，只过滤跟关键字相关的数据
				if(isVerfiy){
					
					if (!TextUtils.isEmpty(from)) {
						
						String verfiyCode = patternCode(patternVerfiyCode,message);
						String code = patternCode(patternCoder,verfiyCode);
						
						if (!TextUtils.isEmpty(code)) {
							Message msg=new Message();
							msg.what=1;
							Bundle bundle=new Bundle();
							bundle.putString("messagecode", code);
							msg.setData(bundle);
							handler.sendMessage(msg);
						}
					}
				}
				
				
			}
		}
		
	}
	
	/**
	 * 获取正则表达式匹配的结果
	 * @param regex
	 * @param patternContent
	 * @return
	 */
	private String patternCode(String regex,String patternContent) {
		if (TextUtils.isEmpty(patternContent)) {
			return null;
		}
		Pattern p = Pattern.compile(regex);
		Matcher matcher = p.matcher(patternContent);
		if (matcher.find()) {
			return matcher.group();
		}
		return null;
	}
	
	/**
	 * 验证是否与正则表达式一致
	 * @param regex
	 * @param str
	 * @return
	 */
	private boolean match(String regex, String str){
		Pattern pattern = Pattern.compile(regex);
		Matcher matcher = pattern.matcher(str);
		return matcher.matches();
	}
	
	
	private Handler handler = new Handler() {
		public void handleMessage(android.os.Message msg) {
			switch (msg.what) {
			case 1:
				String codeMsg=msg.getData().getString("messagecode");
				VerfyCodeFeatureImpl.this.webView.executeScript("javascript:setVerfyCode('"+codeMsg+"')");
				Log.d("得到的code", codeMsg);
				break;
			default:
				break;
			}
			
		};
	};
	
	

}
